# -*- coding: utf-8 -*-
from __future__ import print_function
# (c)opyright L. Humbert, 11., 10. September, 18., 19. August 2009, 
# 16. April 2012
# letzte Änderungen 10. Januar 2015 -- Anpassungen wg. 
# Python 3.x
#
# $Id: kryptologie.py 1184 2015-04-15 12:44:15Z humbert $
#
# Sammlung einiger Funktionen, die im Zusammenhang mit der Kryptologie 
# nuetzlich sind
# --
# einige Funktionen wurden in Anlehnung an die Quelle
#   Grischott2008
# gebaut
#
#
alphabet= "abcdefghijklmnopqrstuvwxyz"
#          01234567890123456789012345 <==

# vigenere-Quadrat -- wird genau einmal beim Laden des Moduls gefuellt

quadrat= []
liste = [chr(zeichennummer) for zeichennummer in range(ord("A"), ord("A")+26)]
quadrat.append(liste)
for naechstes in range(1,26):
    quadrat.append(liste[naechstes:]+liste[0:naechstes])

def vigenere(eingabetext="",schluesselwort=""):
    """fuer die Rueckgabe werden Bloecke zu je 5 Zeichen zusammengestellt"""
    ordA=ord("A")
    eingabetext = eingabetext.upper()
    schluesselwort= schluesselwort.upper()
    schluessellaenge= schluesselwort.__len__() # objektorientierte Schreibweise
    aktuelleNummer=0
    rueckgabe=""
    for zeichen in eingabetext:
        if zeichen != " ":
            rueckgabe += quadrat[ord(zeichen)-ordA][ord(schluesselwort[aktuelleNummer % schluessellaenge])-ordA]
            aktuelleNummer +=1
            if aktuelleNummer % 5 == 0: 
                rueckgabe= rueckgabe + " "
    return rueckgabe


def verschiebe(eingabetext="",verschiebung=0):
    # einfache monoalphabetische Verschlusselung/Entschluesselung
    # bedient sich der Verschiebeoperation bzgl. des Alphabets
    rueckgabe=""
    for zeichen in eingabetext:
        if zeichen == " ":
            rueckgabe= rueckgabe + " "
        else:
            nummer= alphabet.index(zeichen.lower())
            nummer= nummer + verschiebung
            nummer= nummer % alphabet.__len__()  # gewoehnlich == 26
            rueckgabe= rueckgabe + alphabet[nummer]
    return rueckgabe


from math import sqrt
def faktorisiere(x):
    # stellt in simpelster Weise die Möglichkeit bereit, 
    # eine Zahl in ihre Faktoren zu zerlegen
    faktoren= []
    faktor= 2

    while x%faktor: ## x mod faktor <> 0
        faktor= faktor + 1

    faktoren.append(faktor)
    
    if faktor <= sqrt(x):
        weiterefaktoren= faktorisiere(x/faktor)
        faktoren.extend(weiterefaktoren)

    return faktoren


def divalgorithmus(divident,divisor):
    # implementiert den Divisionsalgorithmus
    # fortlaufende Division mit Rest
    # Rueckgabe als Liste mit allen Paaren
    try:
        t=len(rueck)
    except:
        rueck=list()        
    if divisor==0:
        return "NaN"
    else:
        if divident==0:
            return rueck        
        ndivident,ndivisor=divmod(divident,divisor)
        rueck.append((ndivident,ndivisor))
        rueck.extend(divalgorithmus(ndivident,divisor))        
        return rueck


def horner(x,faktoren):
    # (((faktoren[0]*x + faktoren[1])*x + faktoren[2])*x + ... faktoren[n-1])*x + faktoren[n]
    if faktoren.__len__()>0:
        if faktoren.__len__()==1:
            return faktoren[0]
        else:
            return (horner(x,faktoren[:-1]))*x + faktoren[-1]
        return (horner(x,faktoren[:-1]))*x + faktoren[-1]

def ggt(m,n):
   if m==0:
        return n
   else:
        return ggt(n % m, m)

def ggT(m,n):
    return ggt(m,n)


from math import floor
# hier muss noch eine Aenderung wg. python3 eingebaut werden
# / ist in // zu aendern
def erweiterter_euklid(a,b):
    if b == 0:
        return (a,1,0)
    (dn,sn,tn) = erweiterter_euklid(b, a%b)
    (d,s,t) = (dn,tn,sn - a//b * tn)
    return (int(d),int(s),int(t))


def kgv(m,n):
    return m*n/ggt(m,n)

def kgV(m,n):
    return kgv(m,n)

     
if __name__ == "__main__":
    print ("Verschluesselung nach Caesar")
    print ("----------------------------")
    try:
       input = raw_input
    except NameError:
       pass
    klartext= input("Klartext eingeben: ")
    verschiebung= int(input("Schluesselzahl eingeben: "))

    print  ("Geheimtext:", verschiebe(klartext,verschiebung).upper())
    
    print ("\nDivisionsalgorithmus - fortlaufende Division mit Rest")
    print ("-----------------------------------------------------")
    modul=27
    ns=(10322,9662,19683)
    for n in ns:
        print (n,":",modul, 
          "liefert [(Ganz, Rest),..]", divalgorithmus(n,modul))
    
    print ("\nHornerschema zur Bestimmung von Werten")
    print ("--------------------------------------")
    print ("horner(27,[14,4,8]) liefert:")
    print (horner(27,[14,4,8]))
    print ("horner(27,[7,5,8]) liefert:")
    print (horner(27,[7,5,8]))
    

    print ("\nErweiterter Euklidischer Algorithmus")
    print ("------------------------------------")
    e,l=1721,kgV(307-1,859-1)
    print ("erweiterter_euklid (e=",e,",l= kgV(307-1,859-1)=",l,") liefert:")
    ggt,fak1,fak2=erweiterter_euklid(e,l)
    if fak2<0:
        print (ggt,"=",fak1,"*",e ,fak2,"*",l,end="")
    else:
        print (ggt,"=",fak1,"*",e,"+",fak2,"*",l,end="")
    print ("=> d=",fak1)
    
    
    
